// with some help from:
// https://github.com/muaz-khan/WebRTC-Experiment/blob/master/demos/client-side-datachannel.html#L238

const {Cu} = require("chrome");
Cu.import('resource://gre/modules/Services.jsm');

var hiddenWindow = Services.appShell.hiddenDOMWindow;
mozRTCPeerConnection = hiddenWindow.mozRTCPeerConnection;
mozRTCSessionDescription = hiddenWindow.mozRTCSessionDescription;
mozRTCIceCandidate = hiddenWindow.mozRTCIceCandidate;
Blob = hiddenWindow.Blob;

// converts an arraybuffer to a string.
function stoa(s) {
  var buffer = new ArrayBuffer(s.length);
  var bytes = new Uint8Array(buffer);
  for (var i = 0; i < s.length; ++i) {
    bytes[i] = s.charCodeAt(i);
  }
  return buffer;
}

var config = {
  iceServers: [{url: 'stun:stun.l.google.com:19302'}]
};

var offerer = new mozRTCPeerConnection(config, undefined);
var answerer = new mozRTCPeerConnection(config, undefined);

offerer.onicecandidate = function (event) {
  if (!event || !event.candidate) return;
  answerer.addIceCandidate(event.candidate);
};
answerer.onicecandidate = function (event) {
  if (!event || !event.candidate) return;
  offerer.addIceCandidate(event.candidate);
};

var channel = offerer.createDataChannel('chan');
channel.onopen = function () {
  console.info('created datachannel, sending arraybuffer message...');
  channel.send(stoa('hello world!'));
};
channel.onerror = function (e) {
  console.error('error creating datachannel: ' + e.message);
};

offerer.createOffer(function (offerSdp) {
  offerer.setLocalDescription(offerSdp);

  answerer.setRemoteDescription(offerSdp);
  answerer.createAnswer(function (answerSdp) {
    answerer.setLocalDescription(answerSdp);
    offerer.setRemoteDescription(answerSdp);
  }, function(e) {
    console.error('could not create answer: ' + e.message);
  });
}, function(e) {
  console.error('could not create offer: ' + e.message);
});

answerer.ondatachannel = function(event) {
  var channel = event.channel;
  channel.binaryType = 'arraybuffer';
  channel.onmessage = function(e) {
    if (e.data instanceof String) {
      console.info('received string!');
    } else if (e.data instanceof ArrayBuffer) {
      console.info('received arraybuffer!');
    } else if (e.data instanceof Blob) {
      console.info('received blob!');
    } else {
      console.error('received unknown message type');
    }
  };
}
